""" In the night, the mould attacks. """

import pygame.surface
import pygame.locals as pgl

import pymunk

from .base import BaseScene
from ..battery import BatteryManager
from ..lights import LightManager
from ..infobar import InfoBar, CountDownBar
from ..obstacles import ObstacleManager
from ..enemies import Boyd
from ..events import SceneChangeEvent
from ..utils import debug_timer, shadowed_text
from ..loader import loader
from ..sound import sound
from ..transforms import Overlay
from ..turnip import Turnip
from ..widgets import ImageButton
from ..constants import (
    NIGHT_LENGTH, NIGHT_HOURS_PER_TICK, DEBUG, FONTS,
    SCREEN_SIZE, FPS)


class NightScene(BaseScene):

    DARKNESS = Overlay(colour=(0, 0, 0, 150))

    def enter(self, gamestate):
        self._space = pymunk.Space()
        self._obstacles = ObstacleManager(self._space, gamestate)
        self._lights = LightManager(self._space, gamestate)
        self._battery = BatteryManager(gamestate)
        self._battery_dead = False
        self.check_battery()
        self._infobar = InfoBar("day", battery=self._battery, scene=self)
        self._countdownbar = CountDownBar("h")
        self._mould = Boyd(gamestate, self._space)
        self._turnips = []
        for turnip_data in gamestate.turnips:
            turnip = Turnip(space=self._space, **turnip_data)
            self._turnips.append(turnip)
        self._soil = loader.load_image(
            "textures", "soil.png", transform=self.DARKNESS)
        self._tools = self.create_tools(gamestate)
        self._total_ticks = 0
        self._do_ticks = True
        self._paused = False
        self._eaten_tonight = 0
        self._night_over_text = []
        self._ending = False

    def create_tools(self, gamestate):
        tools = []
        y = SCREEN_SIZE[1] - 40
        tools.append(ImageButton(
            '32', 'pause.png', name='pause play',
            pos=(SCREEN_SIZE[0] - 150, y)))
        tools.append(ImageButton(
            '32', 'exit.png', name='exit', pos=(SCREEN_SIZE[0] - 50, y)))
        return tools

    def add_day_button(self):
        y = SCREEN_SIZE[1] - 40
        self._tools.append(ImageButton(
            '32', 'day.png', name='day', pos=(SCREEN_SIZE[0] - 200, y)))

    @property
    def turnip_count(self):
        return len(self._turnips)

    @property
    def power_usage(self):
        power = self._lights.total_power_usage()
        power = power / (FPS * NIGHT_HOURS_PER_TICK)
        return int(round(power))

    def remaining_hours(self):
        return int(round(
            (NIGHT_LENGTH - self._total_ticks) * NIGHT_HOURS_PER_TICK))

    @debug_timer("night.render")
    def render(self, surface, gamestate):
        surface.blit(self._soil, (0, 0))

        self._mould.render(surface)

        for turnip in self._turnips[:]:
            if turnip.eaten:
                self._turnips.remove(turnip)
                turnip.remove()
                gamestate.eaten += 1
                self._eaten_tonight += 1
            else:
                turnip.render(surface)

        self._lights.render_light(surface)
        self._obstacles.render(surface)
        self._lights.render_fittings(surface)
        self._infobar.render(surface, gamestate)
        self._countdownbar.render(surface, self.remaining_hours())

        for tool in self._tools:
            tool.render(surface)

        for text, text_pos in self._night_over_text:
            surface.blit(text, text_pos, None)

    def event(self, ev, gamestate):
        if self._ending:
            return
        if ev.type == pgl.KEYDOWN:
            if not self._do_ticks:
                # Any keypress exits
                self._to_day(gamestate)
            if ev.key in (pgl.K_q, pgl.K_ESCAPE):
                self._ending = True
                from .menu import MenuScene
                SceneChangeEvent.post(scene=MenuScene())
            elif ev.key == pgl.K_e and DEBUG:
                self._end_night()
            elif ev.key == pgl.K_SPACE:
                self.toggle_pause()
        elif ev.type == pgl.MOUSEBUTTONDOWN:
            if not self._do_ticks:
                # Any mouse press exits
                self._to_day(gamestate)
            if ev.button == 1:
                self._lights.toggle_nearest(ev.pos, surfpos=True)

                # Check tools
                for tool in self._tools:
                    if tool.pressed(ev):
                        if tool.name == 'pause play':
                            self.toggle_pause()
                        elif tool.name == 'exit':
                            self._ending = True
                            from .menu import MenuScene
                            SceneChangeEvent.post(scene=MenuScene())
                        elif tool.name == 'day':
                            self._to_day(gamestate)

    def toggle_pause(self):
        self._paused = not self._paused
        pause_img = "play.png" if self._paused else "pause.png"
        for tool in self._tools:
            if tool.name == 'pause play':
                tool.update_image("32", pause_img)

    def _to_day(self, gamestate):
        # End the night
        if self._ending:
            return
        gamestate.update_lights(self._lights)
        # Turnip
        self.grow_turnips(gamestate)
        turnip_data = [turnip.serialize() for turnip in self._turnips]
        gamestate.turnips = turnip_data
        gamestate.days += 1
        self._mould.update_resistances(gamestate)
        self._ending = True
        from .day import DayScene
        SceneChangeEvent.post(scene=DayScene())

    def _end_night(self):
        self._do_ticks = False
        self._night_over_text = []
        overlay = pygame.surface.Surface(
            (SCREEN_SIZE[0], 240), pgl.SWSURFACE).convert_alpha()
        overlay.fill((0, 0, 0, 172))
        self._night_over_text.append((overlay, (0, 40)))
        self._night_over_text.append(
            (shadowed_text("The Night is Over", FONTS["bold"], 48), (300, 50)))
        self._night_over_text.append(
            (shadowed_text("Turnips eaten tonight: %d" % self._eaten_tonight,
                           FONTS["sans"], 32), (300, 130)))
        self._night_over_text.append(
            (shadowed_text("Surviving turnips: %d" % len(self._turnips),
                           FONTS["sans"], 32), (300, 170)))
        self._night_over_text.append(
            (shadowed_text("Press any key to continue", FONTS["sans"], 24),
             (350, 240)))

    def check_battery(self):
        if self._battery.current == 0 and not self._battery_dead:
            self._battery_dead = True
            sound.play_sound("battery_dying.ogg")
            self._lights.battery_dead()

    @debug_timer("night.tick")
    def tick(self, gamestate):
        if self._do_ticks and not self._paused:
            if self._total_ticks < NIGHT_LENGTH:
                self._mould.tick(gamestate, self._space, self._lights)
                self._lights.tick()
                if self._total_ticks % FPS == 0:
                    self._battery.current -= int(
                        self._lights.total_power_usage())
                    self.check_battery()
                self._total_ticks += 1
            else:
                self._end_night()
            if not self._mould.alive():
                self._end_night()
            if not self.turnip_count:
                self.add_day_button()
            if not self.turnip_count and not self._battery.current:
                self._end_night()

    def grow_turnips(self, gamestate):
        """ Turnips grow at the end of the night """
        for turnip in self._turnips[:]:
            # Turnips grow at dawn
            seeds = turnip.grow()
            if seeds:
                gamestate.seeds += seeds
                gamestate.harvested += 1
                self._turnips.remove(turnip)
                # We ignore the body cleanup, since the space is going away
